''*****************************
''*  16Colour Bitmap          *
''*  (C) 2009 Jim Bagley      *
''*  + some animation demos   *
''*     Peter Jakacki         *
''*****************************

CON

{Hybrid Setup
_clkmode = xtal1 + pll16x
_xinfreq = 6_000_000
modepins = %011_0000
}

'{Proto/Demo Board Setup
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  modepins = %001_0101
'}

{Hydra Setup
_clkmode = xtal1 + pll8x
_xinfreq = 10_000_000
modepins = %011_0000
'}

PAL=%0001
NTSC=%0000
INTERLACED=%0010

x_tiles = 8 '*16=256            ' 128 bytes/line
y_tiles = 12 '*16=192
mode = 0
my_hx = 22+((mode&1)<<1)

x_pixels = x_tiles*16
y_pixels = y_tiles*16

bcnt = 128*191
clockfreq = ((_CLKMODE - XTAL1) >> 6) * _XINFREQ
_1mS  = clockfreq /     1_000 'Divisor for  1mS

  maxSprites    = 8
  maxYSZ        = 24
  maxXSZ        = 16
  spriteSZ      = maxYSZ*maxXSZ

  #0,MrHeli,Missile,BadHeli

OBJ
  tv    : "JB_TV_AMS_01"

VAR
  long  frame

  byte  Sprites[16*maxYSZ*maxSprites]
  byte  Shadows[16*maxYSZ*maxSprites]

  byte  SpriteXSZ[maxSprites]
  byte  SpriteYSZ[maxSprites]
  byte  shadowX[maxSprites]
  byte  shadowY[maxSprites]


PUB Main | i,ix[8],iy[8],ii[8],c,x,y,lc

  tv.start(@tvparams)
  frame~
  ' Grab areas of the screen to make some sprites
  MakeSprite(MrHeli,58,106,12,22)
  MakeSprite(Missile,104,113,6,5)
  MakeSprite(BadHeli,0,110,8,14)

  ' now zap this area of the screen for a blank playing field
  bytefill(screen(0,96),0,(158-96)<<6)

  ' Initial sprite positions
  ix[MrHeli] := 56
  iy[MrHeli] := 106
  ix[Missile] := 104
  iy[Missile] := 114
  iy[BadHeli] := 64
  ' initial speeds
  ii[MrHeli] := 2
  ii[BadHeli] := 2

  repeat
    VSync
    'RestoreScreen
    repeat y from 159 to 191    ' scroll this horizontal area
      i := @data+(y<<6)         ' point to line in memory
      c := byte[ i ]            ' save the first byte as we are scrolling left
      bytemove(i,i+1,63)        ' move the line left 2 pixels
      byte[i+63] := c           ' and wrap-around with the original left pixels now on the right
    Vsync                       ' all this takes time, take a breather and resynch
    repeat i from 0 to 2        ' update the sprites
      DrawSprite(i,ix[i],iy[i])

    Vsync
    ' movement update for all the sprites
    ix[missile] += 2
    ix[missile] &= 127

    ix[MrHeli] += 2
    iy[MrHeli] += ii
    if iy[MrHeli] > 159-maxYSZ
      ii[MrHeli] := -2
    if iy[MrHeli] == 70
      ii[MrHeli] := 2
    ix[MrHeli] &= 127

    if lc&1 == 0                ' slower updates
      ix[BadHeli] +=2           ' Moving forward
      ix[BadHeli] &= 127
      iy[BadHeli] += ii[BadHeli] ' vertical movement
      -ii[BadHeli]              ' bounce the heli up and down

    lc++                        ' loop counter

pub MakeSprite(n,x,y,xsz,ysz) | i
  spriteXSZ[n] := xsz           ' save size properties
  spriteYSZ[n] := ysz
  repeat i from 0 to ysz        ' Q&D screen area grab to sprite table
     bytemove(@sprites[(n*spriteSZ)+(i*maxYSZ)], @data+(x>>1)+((y+i)<<6), xsz>>1)

pub DrawSprite(n,x,y) | i
  x &= 127                      ' wrap any X given as it may be incrementing past the right side
  RestoreShadow(n)              ' undraw previous sprite position
  SaveShadow(n,x,y)             ' save this new area of the screen that the sprite will overwrite
  repeat i from 0 to spriteYSZ[n] ' Q&D screen area write from sprite table
    bytemove(screen(x,y+i), @sprites[(n*spriteSZ)+(i*maxYSZ)], spriteXSZ[n]>>1)


pub SaveShadow(n,x,y) | i,p
  shadowX[n] := x               ' remember the position of the shadow for later restoration
  shadowY[n] := y
  p := n*spriteSZ               ' speed things up a bit and remember sprite table address
  repeat i from 0 to spriteYSZ[n]
     bytemove(@shadows[p+(i*maxXSZ)], screen(x,y+i), spriteXSZ[n]>>1)

pub RestoreShadow(n) | i,p
  if shadowx[n] + shadowy[n]    ' another Q&D to prevent restore the first time the sprite is drawn
    p := n*spriteSZ
    repeat i from 0 to spriteYSZ[n]
       bytemove(screen(shadowX[n], shadowY[n]+i), @shadows[p+(i*maxXSZ)], spriteXSZ[n]>>1)

pub RestoreScreen | i           ' part of the "draw em all, restore them all" plan for later
  repeat i from 0 to maxSprites-1
    RestoreShadow(i)

pub screen(x,y)                 ' point to the screen memory which consists of 2 pixels/byte, 64 bytes/line
  result := @data+(x>>1)+(y<<6)


PUB plot(x,y,c)
  if(x<0 or x=>x_pixels or y<0 or y=>y_pixels)
    return
  c&=15
  if(x&1)
    data[y<<6+x>>1]:=(data[y<<6+x>>1]&$f0)+c
  else
    data[y<<6+x>>1]:=(data[y<<6+x>>1]&$0f)+(c<<4)
    
PUB VSync
  repeat until VSyncFlag<>1
  repeat until VSyncFlag==1

PUB ms(Period)
  if Period                                    '1 mS
    waitcnt(_1mS * Period + cnt)

DAT

tvparams      long    0               'status
              long    1               'enable
              long    modepins        'pins ' PROTO/DEMO BOARD = %001_0101 ' HYDRA = %011_0000
              long    mode            'mode
              long    x_tiles         'hc
              long    y_tiles         'vc
              long    my_hx           'hx
VSyncFlag     long    0
nextline      long    0                        
disp_ptr      long    @data+$10
bordercolour  long    $02
              long    @palette+$10

palette       byte    $02,$07,$28,$8d,$ba,$1a,$db,$3d,$8f,$5f,$3f,$5b,$d8,$04,$02,$02

data          file    "mrheli.bit"



